/**********************************************/
/* integral.c:                                */
/* Copyright (C) 2003 Tomonori Kouya          */
/*                                            */
/* This library is free software; you can re- */
/* distribute it and/or modify it under the   */
/* terms of the GNU Lesser General Public     */
/* License as published by the Free Software  */
/* Foundation; either version 2.1 of the      */
/* License, or (at your option) any later     */
/* version.                                   */
/*                                            */
/* This library is distributed in the hope    */
/* that it will be useful, but WITHOUT ANY    */
/* WARRANTY; without even the implied         */
/* warranty of MERCHANTABILITY or FITNESS FOR */
/* A PARTICULAR PURPOSE.  See the GNU Lesser  */
/* General Public License for more details.   */
/**********************************************/
#include <stdio.h>
#include "bnc.h"

/* float: Trapezoidal rule */
float ftrapezoidal_fs(float x_start, float x_end, float (*func)(float x), long int num_div)
{
	float x, h, ret;
	long int i;

	/* set stepsize */
	h = (x_end - x_start) / num_div;

	/* ret := (f(a) + f(b))/2 */
	ret = (func(x_start) + func(x_end)) / 2;

	/* ret += sum^{n-1}_{i = 1} f(x_i) */
	for(i = 1; i < num_div; i++)
	{
		x = x_start + h * i;
		ret += func(x);
	}
	ret *= h;

	return ret;
}

/* double: Trapezoidal rule */
double dtrapezoidal_fs(double x_start, double x_end, double (*func)(double x), long int num_div)
{
	double x, h, ret;
	long int i;

	/* set stepsize */
	h = (x_end - x_start) / num_div;

	/* ret := (f(a) + f(b))/2 */
	ret = (func(x_start) + func(x_end)) / 2;

	/* ret += sum^{n-1}_{i = 1} f(x_i) */
	for(i = 1; i < num_div; i++)
	{
		x = x_start + h * i;
		ret += func(x);
	}
	ret *= h;

	return ret;
}

#ifdef USE_GMP
/* Trapezoidal rule */
void mpf_trapezoidal_fs(mpf_t ret, mpf_t x_start, mpf_t x_end, void (*func)(mpf_t, mpf_t), long int num_div)
{
	mpf_t x, h, tmp;
	long int i;

	mpf_init2(x, mpf_get_prec(x_start));
	mpf_init2(h, mpf_get_prec(x_start));
	mpf_init2(tmp, mpf_get_prec(ret));

	/* set Stepsize */
	mpf_sub(h, x_end, x_start); mpf_div_ui(h, h, (unsigned long)num_div);

	/* ret := (f(a) + f(b)) / 2 */
	func(ret, x_start);
	func(tmp, x_end);
	mpf_add(ret, ret, tmp); mpf_div_ui(ret, ret, 2UL);

	/* set starting and ending values */
	mpf_add(x, x_start, h);

	/* ret += sum^{n-1}_{i=1} f(x_i) */
	for(i = 0; i < num_div - 1; i++)
	{
		func(tmp, x);
		mpf_add(ret, ret, tmp);
		mpf_add(x, x, h);
	}

	/* ret := h * { (f(a) + f(b)) / 2 + sum^{n-1}_{i=1} f(x_i) } */
	mpf_mul(ret, ret, h);

	mpf_clear(x);
	mpf_clear(h);
	mpf_clear(tmp);
}
#endif
